package ru.dbotthepony.kstarbound.defs

import com.google.gson.Gson
import com.google.gson.TypeAdapter
import com.google.gson.annotations.JsonAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kommons.io.readBinaryString
import ru.dbotthepony.kommons.io.writeBinaryString
import ru.dbotthepony.kstarbound.Registries
import ru.dbotthepony.kstarbound.Registry
import ru.dbotthepony.kstarbound.io.readNullableDouble
import ru.dbotthepony.kstarbound.io.writeNullableDouble
import ru.dbotthepony.kstarbound.json.builder.FactoryAdapter
import java.io.DataInputStream
import java.io.DataOutputStream

@JsonAdapter(EphemeralStatusEffect.Adapter::class)
data class EphemeralStatusEffect(val effect: Registry.Ref<StatusEffectDefinition>, val duration: Double? = null) {
	constructor(stream: DataInputStream, isLegacy: Boolean) : this(Registries.statusEffects.refOrThrow(stream.readBinaryString()), stream.readNullableDouble())
	constructor(effect: Registry.Entry<StatusEffectDefinition>, duration: Double? = null) : this(effect.ref, duration)

	class Adapter(gson: Gson) : TypeAdapter<EphemeralStatusEffect>() {
		private val factory = FactoryAdapter.createFor(EphemeralStatusEffect::class, gson = gson)

		override fun write(out: JsonWriter, value: EphemeralStatusEffect) {
			if (value.duration == null)
				out.value(value.effect.key.left())
			else
				factory.write(out, value)
		}

		override fun read(`in`: JsonReader): EphemeralStatusEffect {
			if (`in`.peek() == JsonToken.STRING)
				return EphemeralStatusEffect(Registries.statusEffects.ref(`in`.nextString()))
			else
				return factory.read(`in`)
		}
	}

	fun write(stream: DataOutputStream, isLegacy: Boolean) {
		stream.writeBinaryString(effect.key.left())
		stream.writeNullableDouble(duration, isLegacy)
	}
}
