package ru.dbotthepony.kstarbound.io

import ru.dbotthepony.kommons.io.writeVarInt
import java.io.InputStream
import java.io.OutputStream
import java.util.*

private fun toBytes(key: UUID): ByteArray {
	val value = ByteArray(16)

	value[0] = ((key.mostSignificantBits ushr 56) and 0xFFL).toByte()
	value[1] = ((key.mostSignificantBits ushr 48) and 0xFFL).toByte()
	value[2] = ((key.mostSignificantBits ushr 40) and 0xFFL).toByte()
	value[3] = ((key.mostSignificantBits ushr 32) and 0xFFL).toByte()
	value[4] = ((key.mostSignificantBits ushr 24) and 0xFFL).toByte()
	value[5] = ((key.mostSignificantBits ushr 16) and 0xFFL).toByte()
	value[6] = ((key.mostSignificantBits ushr 8) and 0xFFL).toByte()
	value[7] = ((key.mostSignificantBits ushr 0) and 0xFFL).toByte()

	value[8 + 0] = ((key.leastSignificantBits ushr 56) and 0xFFL).toByte()
	value[8 + 1] = ((key.leastSignificantBits ushr 48) and 0xFFL).toByte()
	value[8 + 2] = ((key.leastSignificantBits ushr 40) and 0xFFL).toByte()
	value[8 + 3] = ((key.leastSignificantBits ushr 32) and 0xFFL).toByte()
	value[8 + 4] = ((key.leastSignificantBits ushr 24) and 0xFFL).toByte()
	value[8 + 5] = ((key.leastSignificantBits ushr 16) and 0xFFL).toByte()
	value[8 + 6] = ((key.leastSignificantBits ushr 8) and 0xFFL).toByte()
	value[8 + 7] = ((key.leastSignificantBits ushr 0) and 0xFFL).toByte()

	return value
}

class ByteKey private constructor(private val bytes: ByteArray, mark: Nothing?) : Comparable<ByteKey> {
	constructor(vararg bytes: Byte) : this(bytes, null)
	constructor(key: String) : this(key.toByteArray(), null)
	constructor(key: UUID) : this(toBytes(key), null)

	override fun equals(other: Any?): Boolean {
		return this === other || other is ByteKey && other.bytes.contentEquals(bytes)
	}

	val size: Int
		get() = bytes.size

	operator fun get(index: Int): Byte {
		return bytes[index]
	}

	fun write(stream: OutputStream) {
		stream.writeVarInt(bytes.size)
		stream.write(bytes)
	}

	fun writeRaw(stream: OutputStream) {
		stream.write(bytes)
	}

	fun toByteArray(): ByteArray {
		return bytes.clone()
	}

	override fun hashCode(): Int {
		return bytes.contentHashCode()
	}

	override fun toString(): String {
		return "ByteKey[${bytes.map { it.toInt() and 0xFF }.joinToString(", ")}]"
	}

	override fun compareTo(other: ByteKey): Int {
		val cmp = size.compareTo(other.size)
		if (cmp != 0) return cmp

		for (i in bytes.indices) {
			if (bytes[i].toInt() and 0xFF > other.bytes[i].toInt() and 0xFF) {
				return 1
			} else if (bytes[i].toInt() and 0xFF < other.bytes[i].toInt() and 0xFF) {
				return -1
			}
		}

		return 0
	}

	companion object {
		/**
		 * Constructs [ByteKey] without any copying of provided array
		 */
		@JvmStatic
		fun wrap(bytes: ByteArray): ByteKey {
			return ByteKey(bytes, null)
		}

		@JvmStatic
		fun read(stream: InputStream): ByteKey {
			return stream.readByteKey()
		}

		@JvmStatic
		fun readRaw(stream: InputStream, size: Int): ByteKey {
			return stream.readByteKeyRaw(size)
		}
	}
}